#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "unistd.h"
#include "sys/types.h"

#define UID_BUF_SIZE 10

/// Version 0.02

/// This program will write a BOINC cc_config.xml file with the two options "report_results_immediately",
/// and "no_gpus". It does not actually understand XML; everything is just text, so it's not very flexible.
/// It will most likely need to be run with sudo to create or modify the file.
/// WARNING: Any previous custom configuration will be overwritten.


int write_config(char *filename, unsigned short report_immediately, unsigned short no_gpus_in)
{
    FILE *file = NULL;
    file = fopen(filename, "w");

    if(file == NULL)	/* Check again to be sure. */
        exit(EXIT_FAILURE);

    /* Use sprintf to convert the numerical variables to strings. Probably useless, but ushorts
       can be up to 5 digits long, so make the strings 5 bytes just in case. */
    char report_immediately_str[5];
    sprintf(report_immediately_str, "%d", report_immediately);
    char no_gpus_str[5];
    sprintf(no_gpus_str, "%d", no_gpus_in);

	/* Manually write the XML file as text. */
    fputs("<cc_config>\n\t<options>\n", file);

    fputs("\t\t<report_results_immediately>", file);
    fputs(report_immediately_str, file);
    fputs("</report_results_immediately>\n", file);

    fputs("\t\t<no_gpus>", file);
    fputs(no_gpus_str, file);
    fputs("</no_gpus>\n", file);

    fputs("\t</options>\n</cc_config>\n", file);

    fclose(file);

    return 0;
}

void set_boinc_user(void)
{
    FILE *uid_file;
    char *uid_buffer = (char *)malloc(UID_BUF_SIZE * sizeof(char));	/* Buffer to hold the boinc user id */

    uid_file = popen("id -u boinc", "r");		/* popen returns a file pointer instead of a string. */
    fgets(uid_buffer, UID_BUF_SIZE, uid_file);	/* Read the buffer returned by the id command */

    uid_t boinc_uid = (uid_t)strtol(uid_buffer, NULL, 10);		/* Convert the boinc uid to an int (uid_t) */

    pclose(uid_file);	/* Files pointers returned by popen are closed with pclose */
    free(uid_buffer);

    if(setuid(boinc_uid) != 0)		/* Try to become the boinc user */
    {
        printf("Error: could not run as boinc user. Use sudo?\n");
        exit(EXIT_FAILURE);
    }
}

int main(int argc, char *argv[])
{
	set_boinc_user();

    char *out_filename;
    const char *default_filename = "/etc/boinc-client/cc_config.xml";

    if(argc == 1)		/* If there are no arguments, use the default path. */
    {
        out_filename = (char *)default_filename;
        printf("Warning: using default boinc config path %s\n\n", default_filename);
    }
    else if(argc == 2)	/* Assume the first argument is the filename. */
    {
    	if(strstr(argv[1], "cc_config.xml") == NULL)	/* For security, only allow cc_config.xml for filename. */
		{
			printf("Error: file name must be cc_config.xml\n");
			exit(EXIT_FAILURE);
		}
    	else
		{
			out_filename = argv[1];
		}
    }
    else	/* Print usage if there is more than one argument. */
    {
        printf("Invalid arguments.\n Usage: ./%s (will use %s as default path)\n", argv[0], default_filename);
        printf(" Or     ./%s \'path-to-cc_config.xml\'\n", argv[0]);
        exit(EXIT_FAILURE);
    }

	/* Check to see if the file is writable before asking any questions. */

    FILE *test_file = fopen(out_filename, "a");
    if(test_file == NULL)
	{
		printf("Error opening file for writing: \'%s\'\n", out_filename);
		exit(EXIT_FAILURE);
	}
	else
	{
		fclose(test_file);
	}

    unsigned short report_results_immediately = 0;	/* Default is 0 (disabled) */
    unsigned short no_gpus = 0;						/* Default is 0 (GPUs enabled) */

    /* Yes or no? to hold y/n */
    char c;

	/* Ask questions */

    printf("Should BOINC report tasks immediately after they finish? (y/n): ");
    c = getchar();
    printf("\n");

    switch(c)
    {
    case 'y':
        report_results_immediately = 1;
        break;
    case 'n':
        report_results_immediately = 0;
        break;
    default:
        report_results_immediately = 0;
    }

	/* Hack to keep the second question from getting eaten.
	   If someone enters more than one character the first time, it will get eaten anyway. */
	getchar();

	printf("Should BOINC use GPUs for processing if available? (y/n): ");
    c = getchar();
    printf("\n");

	switch(c)
    {
    case 'y':
        no_gpus = 0;
        break;
    case 'n':
        no_gpus = 1;
        break;
    default:
        no_gpus = 0;
    }

	/* Try writing the values to the config file. */
    if(write_config(out_filename, report_results_immediately, no_gpus) == 0)
    {
        printf("Config file written successfully.\n");
    }
    else
    {
        exit(EXIT_FAILURE);
    }

    return 0;
}
